/* Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
* Example that counts frequency of an input signal. Input signal is detected with a GPIOTE channel and
* that increments the counter value of TIMER0 peripheral which is in counter mode. GPIOTE IN channel
* connects with the timer with a PPI channel. A LED is flashed with a lower frequency than the input signal.
 */

#include <stdbool.h>
#include "nrf.h"
#include "nrf_gpio.h"
#include "boards.h"
#include "nrf_gpiote.h"

#ifdef BSP_BUTTON_0
    #define PIN_IN BSP_BUTTON_0
#endif
#ifndef PIN_IN
    #error "Please indicate input pin"
#endif

#ifdef BSP_LED_0
    #define PIN_OUT BSP_LED_0
#endif
#ifndef PIN_OUT
    #error "Please indicate output pin"
#endif

void start_timer(void)
{		
  NRF_TIMER0->MODE = TIMER_MODE_MODE_Counter;              // Set the timer in Counter Mode
  NRF_TIMER0->TASKS_CLEAR = 1;                             // clear the task first to be usable for later
	NRF_TIMER0->BITMODE = TIMER_BITMODE_BITMODE_32Bit;		   //Set counter to 32 bit resolution
	NRF_TIMER0->CC[0] = 500000;                               //Set value for TIMER0 compare register 0
	NRF_TIMER0->CC[1] = 400000;                                   //Set value for TIMER0 compare register 1
		
  // Enable interrupt on Timer 2, both for CC[0] and CC[1] compare match events
	NRF_TIMER0->SHORTS |= (TIMER_SHORTS_COMPARE0_CLEAR_Enabled << TIMER_SHORTS_COMPARE0_CLEAR_Pos);
	NRF_TIMER0->INTENSET = (TIMER_INTENSET_COMPARE0_Enabled << TIMER_INTENSET_COMPARE0_Pos) | (TIMER_INTENSET_COMPARE1_Enabled << TIMER_INTENSET_COMPARE1_Pos);
  NVIC_EnableIRQ(TIMER0_IRQn);
		
  NRF_TIMER0->TASKS_START = 1;               // Start TIMER0
}


/** TIMTER2 peripheral interrupt handler. This interrupt handler is called whenever there it a TIMER2 interrupt
 */
void TIMER0_IRQHandler(void)
{
  if ((NRF_TIMER0->EVENTS_COMPARE[0] != 0) && ((NRF_TIMER0->INTENSET & TIMER_INTENSET_COMPARE0_Msk) != 0))
  {
		NRF_TIMER0->EVENTS_COMPARE[0] = 0;	       //Clear compare register 0 event	
		nrf_gpio_pin_set(PIN_OUT);         //Set LED
  }
	
	if ((NRF_TIMER0->EVENTS_COMPARE[1] != 0) && ((NRF_TIMER0->INTENSET & TIMER_INTENSET_COMPARE1_Msk) != 0))
  {
		NRF_TIMER0->EVENTS_COMPARE[1] = 0;	       //Clear compare register 1 event
		nrf_gpio_pin_clear(PIN_OUT);       //Clear LED
  }
}


/**
 * @brief Function for configuring: PIN_IN pin for input, PIN_OUT pin for output, 
 * and configures GPIOTE to give an interrupt on pin change.
 */
static void gpio_init(void)
{
    nrf_gpio_cfg_output(PIN_OUT);
    nrf_gpio_cfg_input(PIN_IN, NRF_GPIO_PIN_PULLUP);
    nrf_gpio_pin_clear(PIN_OUT);

		nrf_gpiote_event_config(0, PIN_IN, NRF_GPIOTE_POLARITY_TOGGLE);
}


/** @brief Function for initializing Programmable Peripheral Interconnect (PPI) peripheral.
 *   The PPI is needed to convert the timer event into a task.
 */
static void ppi_init(void)
{
    // Configure PPI channel 0 to toggle GPIO_OUTPUT_PIN on every TIMER0 COMPARE[0] match (200 ms)
    NRF_PPI->CH[0].EEP = (uint32_t)&NRF_GPIOTE->EVENTS_IN[0];
		NRF_PPI->CH[0].TEP = (uint32_t)&NRF_TIMER0->TASKS_COUNT;

    // Enable PPI channel 0
    NRF_PPI->CHEN = (PPI_CHEN_CH0_Enabled << PPI_CHEN_CH0_Pos);
}

/**
 * @brief Function for application main entry.
 */
int main(void)
{
    gpio_init();
		start_timer();
		ppi_init();

    while (true)
    {
			
    }
}


/** @} */
